import { withProfiler } from '@sentry/react'
import React, { useMemo } from 'react'
import { useTranslation } from 'react-i18next'
import { StyleProp, StyleSheet, View, ViewStyle } from 'react-native'

import { getRelativeDateString } from '@holi/core/i18n/helpers/dateHelper'
import useRouting from '@holi/core/navigation/hooks/useRouting'
import { Insight } from '@holi/core/screens/insights/types'
import HoliBox from '@holi/ui/components/atoms/HoliBox'
import { HoliGap } from '@holi/ui/components/atoms/HoliGap'
import { HoliHeading } from '@holi/ui/components/atoms/HoliHeading'
import HoliSkeleton from '@holi/ui/components/atoms/HoliSkeleton'
import HoliText from '@holi/ui/components/atoms/HoliText'
import HoliAvatar from '@holi/ui/components/molecules/HoliAvatar'
import HoliButton from '@holi/ui/components/molecules/HoliButton'
import HoliCard from '@holi/ui/components/molecules/HoliCard'
import HoliUserAvatarPile from '@holi/ui/components/organisms/HoliUserAvatarPile'
import { HoliTheme, useTheme } from '@holi/ui/styles/theme'

interface BaseProps {
  style?: StyleProp<ViewStyle>
}
interface SkeletonProps extends BaseProps {
  loading: true
  insight?: Insight
}
interface Props extends BaseProps {
  loading?: false
  insight: Insight
}

type InsightPreviewBoxProps = SkeletonProps | Props

const InsightPreviewBox: React.FC<InsightPreviewBoxProps> = ({ insight, loading, ...props }) => {
  const { navigate } = useRouting()
  const { t, i18n } = useTranslation()
  const { theme } = useTheme()
  const { colors } = theme
  const styles = useMemo(() => getStyles(theme), [theme])

  if (loading) {
    return (
      <View {...props} style={StyleSheet.flatten([styles.skeletonWrapper, props.style])}>
        <HoliSkeleton aspectRatio={327 / 206} borderRadius={16} height={'auto'} width={'100%'} />

        <HoliGap size="s" />
        <HoliText size="small" selectable style={{ color: colors.brandB300 }} loading />

        <HoliGap size="xs" />
        <HoliHeading level="2" selectable size="s" numberOfLines={3} loading skeletonOptions={{ numberOfLines: 2 }} />

        <HoliGap size="xs" />
        <HoliText size="small" selectable loading />

        <HoliUserAvatarPile loading usersToShow={3} counter collapse style={styles.avatarPile} />
      </View>
    )
  }

  const editors = insight.involvedUsers.data

  return (
    <View {...props}>
      <HoliButton
        label={insight.title}
        transparent
        onPress={() => navigate(`/insights/${insight.id}`)}
        testID="feed-insight-image-link"
      >
        <HoliCard
          imageLabel={insight.title}
          imageUrl={insight.headerImage}
          imageWidth={327}
          imageHeight={218}
          imageAspectRatio={1.75}
          hasBorder
        >
          {!!insight.topic && (
            <HoliBox padding={[0, 0, 8, 0]}>
              <HoliText size="small" bold selectable style={{ color: colors.brandB300 }}>
                {insight.topic.title}
              </HoliText>
            </HoliBox>
          )}

          <HoliHeading level="2" selectable size="s" numberOfLines={3}>
            {insight.title}
          </HoliHeading>

          {!!insight.creator && (
            <HoliBox padding={[8, 0, 0, 0]}>
              <View style={styles.container}>
                <View style={styles.row}>
                  <HoliAvatar
                    imageSrc={insight.creator.avatar}
                    imageBlurhash={insight.creator.avatarBlurhash}
                    label={insight.creator.fullName || t('user.profileView.anonymous')}
                    initials={insight.creator.avatarLabel}
                    defaultColor={insight.creator.avatarDefaultColor}
                    size="micro"
                    testID="insight-curator"
                    style={styles.avatar}
                  />
                  <HoliText size="small" style={styles.textGrey}>
                    {insight.creator.fullName}
                  </HoliText>
                  <HoliText size="large" style={styles.dot}>
                    &middot;
                  </HoliText>
                  <HoliText size="small" style={styles.textGrey}>
                    {getRelativeDateString(insight.datePublished, i18n.language)}
                  </HoliText>
                </View>
              </View>
            </HoliBox>
          )}

          {!!editors.length && (
            <HoliUserAvatarPile
              users={editors}
              usersToShow={3}
              usersTotal={insight.involvedUsers.totalResults}
              counter
              collapse
              style={styles.avatarPile}
            />
          )}
        </HoliCard>
      </HoliButton>
    </View>
  )
}

export default withProfiler(InsightPreviewBox, { name: 'InsightPreviewBox' })

const getStyles = (theme: HoliTheme) =>
  StyleSheet.create({
    mainImage: {
      aspectRatio: 327 / 206,
      borderRadius: 12,
      width: '100%',
      backgroundColor: theme.colors.background30,
    },
    avatar: {
      marginRight: 8,
    },
    dot: {
      color: theme.colors.brandB300,
      marginRight: 4,
      marginLeft: 4,
    },
    avatarPile: {
      marginTop: 12,
    },
    textGrey: {
      color: theme.colors.foreground20,
    },
    container: {
      flex: 1,
    },
    row: {
      flexDirection: 'row',
      alignItems: 'center',
    },
    skeletonWrapper: {
      borderWidth: 1,
      borderStyle: 'solid',
      borderColor: theme.colors.gray200,
      padding: 8,
      paddingBottom: 16,
      borderRadius: 16,
      backgroundColor: theme.colors.background10,
    },
  })
